|----------------------------------------------------------------------------
| Graphics functions
| VGA 640*480*8bit mode, FPGA-based framebuffer with hardware acceleration 
|----------------------------------------------------------------------------

|  pppp pppp pppp pppp pppp pppp pppp pppp RRRR
|   +32  +28  +24  +20  +16  +12   +8   +4   sp

                .include "sys_regs.i"
                .include "gfx_regs.i"
                .include "gfx_fnt.i"
|                .include "_pal.i"

|----------------------------------------------------------------------------
                .macro gfx_wait
                btst    #7,GFX_COMMAND_STATUS_REGISTER
                bne.s  .-6
                .endm
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_load_led_register
                .type   gfx_load_led_register, @function
gfx_load_led_register:
                move.b  7(%sp),TEST_LED_REGISTER
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _gfx_irq7_handler

_gfx_irq7_handler:
                st      _gfx_vertical_retrace_flag
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_reset_vertical_retrace
                .type   gfx_reset_vertical_retrace, @function
gfx_reset_vertical_retrace:
                clr.b   _gfx_vertical_retrace_flag
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_is_vertical_retrace
                .type   gfx_is_vertical_retrace, @function
gfx_is_vertical_retrace:
                move.b   _gfx_vertical_retrace_flag,%d0
                rts                
|----------------------------------------------------------------------------
_gfx_vertical_retrace_flag:
                ds.b    1
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_initialize
                .type   gfx_initialize, @function
gfx_initialize:
                move.b  #0,TEST_LED_REGISTER
                clr.w   OVERLAY_CROSS_POINTER_CTRL_REG
                move.l  #0xFFFFFFFF,GFX_BIT_MASK_REGISTER
                move.b  #0x0F,RAMDAC_PIXEL_MASK_REGISTER
                pea     16
                clr.l   -(%sp)
                pea     _gfx_sys_pal0
                bsr.w   gfx_ramdac_set_rgb_palette
                lea     12(%sp),%sp
                pea     0x0D
                bsr.w   gfx_clear_screen
                clr.l   (%sp)
                bsr.w   gfx_clear_screen
                addq.l  #4,%sp
                move.w  #319,_gfx_x0
                move.w  #239,_gfx_y0

|      pea     256
|      pea     0
 |     pea     PAL4 | 2+,3,4,5,6+,7+,13^,21,39
 |     bsr.w   gfx_ramdac_set_rgb_palette
 |     lea     12(%sp),%sp
 |    move.b  #0xFF,RAMDAC_PIXEL_MASK_REGISTER

                move.w  #0x0080,INTERRUPT_POLL_CLEAR_REGISTER_L |clear irq7
                bsr.w   gfx_reset_vertical_retrace
                ori.w   #0x0080,INTERRUPT_MASK_REGISTER_L |unmask irq7
                rts
_gfx_sys_pal0:
                dc.l    0x00000000,  0x001F001F,  0x00001F1F,  0x1F00001F
                dc.l    0x001F1F1F,  0x001F1F1F,  0x0F0F0F00,  0x003F003F
                dc.l    0x00003F3F,  0x3F00003F,  0x003F3F3F,  0x003F3F3F
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_ramdac_set_rgb_palette
                .type   gfx_ramdac_set_rgb_palette, @function

gfx_ramdac_set_rgb_palette:
                movem.l %d0-%d1/%a0,-(%sp)
                move.w  12+14(%sp),%d1
                beq.s   _gfx_setrgbpal_exit
                cmpi.w  #256,%d1
                bhi.s   _gfx_setrgbpal_exit
                lsl.w   #1,%d1
                add.w   12+14(%sp),%d1
                movea.l 12+4(%sp),%a0
                move.b  12+11(%sp),%d0
                subq.w  #1,%d1
                move.b  %d0,RAMDAC_PALETTE_INDEX_REGISTER
_gfx_wr_pal_loop:
                move.b  (%a0)+,RAMDAC_PALETTE_RAM_REGISTER
                dbf     %d1,_gfx_wr_pal_loop
_gfx_setrgbpal_exit:
                movem.l (%sp)+,%d0-%d1/%a0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_ramdac_set_pixel_mask
                .type   gfx_ramdac_set_pixel_mask, @function

gfx_ramdac_set_pixel_mask:
                move.b  7(%sp),RAMDAC_PIXEL_MASK_REGISTER
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_clear_screen
                .type   gfx_clear_screen, @function

gfx_clear_screen:
                move.l  %d0,-(%sp)
                gfx_wait
                move.b  4+7(%sp),GFX_COLOR_REGISTER
                move.w  #320,GFX_LENGTH_REGISTER
                clr.w   GFX_DESTINATION_X_REGISTER
                move.w  #479,%d0
_gfx_cls_y_loop:
                gfx_wait
                move.w  %d0,GFX_DESTINATION_Y_REGISTER
                move.b  #0xBF,GFX_COMMAND_STATUS_REGISTER
                dbf     %d0,_gfx_cls_y_loop
                move.l  (%sp)+,%d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_set_draw_mask
                .type   gfx_set_draw_mask, @function
gfx_set_draw_mask:
                gfx_wait
                move.l  4(%sp),GFX_BIT_MASK_REGISTER
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_line_to
                .type   gfx_line_to, @function
                .globl  gfx_move_to
                .type   gfx_move_to, @function


|  c... y0.. x0... RRRR
|  +15  +10  +6   sp
gfx_line_to:
                gfx_wait
                move.b  15(%sp),GFX_COLOR_REGISTER
                move.w  _gfx_x0,GFX_X1_REGISTER
                move.w  _gfx_y0,GFX_Y1_REGISTER 
                move.w  6(%sp),GFX_X2_REGISTER 
                move.w  10(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER |load gfx opcode

| y0.. x0.. RRRR
| +10  +6   sp
gfx_move_to:
                move.w  6(%sp),_gfx_x0
                move.w  10(%sp),_gfx_y0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
_gfx_x0:        ds.w    1
_gfx_y0:        ds.w    1

|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_draw_pixel
                .type   gfx_draw_pixel, @function

|  c... y1.. x1.. RRRR
|  +15  +10  +6   sp
gfx_draw_pixel:
                gfx_wait
                move.b  15(%sp),GFX_COLOR_REGISTER
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.w  6(%sp),GFX_X2_REGISTER 
                move.w  10(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_line
                .type   gfx_line, @function

|  c... y2.. x2.. y1.. x1.. RRRR
|  +23  +18  +14  +10  +6   sp
gfx_line:
                gfx_wait
                move.b  23(%sp),GFX_COLOR_REGISTER
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.w  14(%sp),GFX_X2_REGISTER 
                move.w  18(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_rectangle
                .type   gfx_rectangle, @function

|  c... y2.. x2.. y1.. x1.. RRRR
|  +23  +18  +14  +10  +6   sp

gfx_rectangle:
                gfx_wait
                move.b  23(%sp),GFX_COLOR_REGISTER
| top horizontal line
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  14(%sp),GFX_X2_REGISTER 
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.w  10(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                gfx_wait
| right vertical line
                move.w  18(%sp),GFX_Y1_REGISTER 
                move.w  14(%sp),GFX_X1_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                gfx_wait
| bottom horizontal line
                move.w  6(%sp),GFX_X2_REGISTER 
                move.w  18(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                gfx_wait
| left vertical line
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_bar
                .type   gfx_bar, @function
|  c... y2.. x2.. y1.. x1.. RRRR
|  +23  +18  +14  +10  +6   sp

gfx_bar:
                movem.l %d0-%d3/%a0-%a2,-(%sp)
                movea.l #GFX_COMMAND_STATUS_REGISTER,%a0
                movea.l #GFX_Y1_REGISTER,%a1
                movea.l #GFX_Y2_REGISTER,%a2

                move.w  28+10(%sp),%d0
                move.w  28+18(%sp),%d1
                cmp.w   %d0,%d1
                bhs.s   _gfx_bar_1
                exg     %d1,%d0
_gfx_bar_1:
                sub.w   %d0,%d1
                moveq   #-1,%d2
                gfx_wait
                move.l  %d2,GFX_BIT_MASK_REGISTER
                move.b  28+23(%sp),GFX_COLOR_REGISTER
                move.w  28+6(%sp),GFX_X1_REGISTER
                move.w  28+14(%sp),GFX_X2_REGISTER
_gfx_bar_loop:
                gfx_wait
                move.w  %d0,(%a1)
                move.w  %d0,(%a2)
                move.b  #0xB8,(%a0)
                addq.w  #1,%d0
                dbf     %d1,_gfx_bar_loop
                movem.l (%sp)+,%d0-%d3/%a0-%a2
                rts

|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_bar_x2
                .type   gfx_bar_x2, @function

|  c... y2.. x2.. y1.. x1.. RRRR
|  +23  +18  +14  +10  +6   sp

gfx_bar_x2:
                movem.l %d0-%d3/%a0-%a1,-(%sp)
                movea.l #GFX_COMMAND_STATUS_REGISTER,%a0
                movea.l #GFX_DESTINATION_Y_REGISTER,%a1
                move.w  24+10(%sp),%d0        | y1
                move.w  24+18(%sp),%d1        | y2
                cmp.w   %d0,%d1
                bhs.s   _gfx_bar2_1
                exg     %d1,%d0
_gfx_bar2_1:
                sub.w   %d0,%d1
                move.w  24+6(%sp),%d2         | x1
                move.w  24+14(%sp),%d3        | x2
                cmp.w   %d2,%d3
                bhs.s   _gfx_bar2_2
                exg     %d2,%d3
_gfx_bar2_2:
                sub.w   %d2,%d3
                lsr.w   #1,%d3                | len = (x2-x1)/2
                gfx_wait
                move.b  24+23(%sp),GFX_COLOR_REGISTER
                move.w  %d2,GFX_DESTINATION_X_REGISTER
                move.w  %d3,GFX_LENGTH_REGISTER
_gfx_bar2_loop:
                gfx_wait
                move.w  %d0,(%a1)             | y
                move.b  #0xBF,(%a0)
                addq.w  #1,%d0
                dbf     %d1,_gfx_bar2_loop
                movem.l (%sp)+,%d0-%d3/%a0-%a1
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_draw_char
                .type   gfx_draw_char, @function

|chr. mode col. y... x... RRRR
|+23  +18  +15  +10  +6   sp

gfx_draw_char:
                movem.l %d0-%d3/%a0-%a3,-(%sp)
                movea.l #GFX_BIT_MASK_REGISTER,%a1
                movea.l #GFX_DESTINATION_Y_REGISTER,%a2
                movea.l #GFX_COMMAND_STATUS_REGISTER,%a3
                clr.l   %d0
                move.b  32+23(%sp),%d0           | chr
                lsl.w   #3,%d0
                lea     _system_font_5x8,%a0
                adda.l  %d0,%a0
                moveq   #-80,%d3              |0xB0 "draw horiz line" opcode
                moveq   #7,%d1                |bit7 of CMD_STATUS_REG = ready
                moveq   #1,%d0                |loop counter
                gfx_wait
                move.b  32+15(%sp),GFX_COLOR_REGISTER
                move.w  32+6(%sp),GFX_DESTINATION_X_REGISTER  | x
                move.w  32+10(%sp),(%a2)                      | y
                move.w  #5,GFX_LENGTH_REGISTER | = character width
_gfx_chr0_r1:
                move.b  (%a0)+,(%a1)          |bit draw_mask
                btst    %d1,(%a3)             |test for gfx command ready
                bne.s   .-2
                move.b  %d3,(%a3)             |load opcode 0xB0, start op
                addq.w  #1,(%a2)              |Y++
                move.b  (%a0)+,(%a1)
                btst    %d1,(%a3)  
                bne.s   .-2
                move.b  %d3,(%a3)
                addq.w  #1,(%a2) 
                move.b  (%a0)+,(%a1)
                btst    %d1,(%a3)   
                bne.s   .-2
                move.b  %d3,(%a3)   
                addq.w  #1,(%a2)    
                move.b  (%a0)+,(%a1)  
                btst    %d1,(%a3)     
                bne.s   .-2
                move.b  %d3,(%a3)     
                addq.w  #1,(%a2)
                dbf     %d0,_gfx_chr0_r1
                moveq   #-1,%d2
                move.l  %d2,(%a1)
                movem.l (%sp)+,%d0-%d3/%a0-%a3
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_draw_text
                .type   gfx_draw_text, @function

|   *str offs mode c... y... x... RRRR
|   +24  +22  +18  +15  +10  +6   sp

gfx_draw_text:
                movem.l %d0-%d4/%a0-%a4,-(%sp)
                movea.l #GFX_BIT_MASK_REGISTER,%a1
                movea.l #GFX_DESTINATION_Y_REGISTER,%a2
                movea.l #GFX_COMMAND_STATUS_REGISTER,%a4
                move.w  40+6(%sp),%d3         | x
                movea.l 40+24(%sp),%a3        |*string
                gfx_wait
                move.b  40+15(%sp),GFX_COLOR_REGISTER
                move.w  #5,GFX_LENGTH_REGISTER | = character width
_gfx_txt0_u1:
                clr.l   %d0
                move.b  (%a3)+,%d0            | chr
                beq.s   _gfx_txt0_e1
                lsl.w   #3,%d0
                lea     _system_font_5x8,%a0
                adda.l  %d0,%a0      
                move.w  %d3,GFX_DESTINATION_X_REGISTER
                move.w  40+10(%sp),(%a2)      | y
                moveq   #1,%d0
                moveq   #-80,%d4              |0xB0 "draw horiz line" opcode
                moveq   #7,%d1                |bit7 of CMD_STATUS_REG = ready
_gfx_txt0_r1:
                move.b  (%a0)+,(%a1)          |bit draw_mask
                btst    %d1,(%a4)             |test for gfx command ready
                bne.s   .-2
                move.b  %d4,(%a4)             |load opcode 0xB0, start op
                addq.w  #1,(%a2)              |Y++
                move.b  (%a0)+,(%a1)
                btst    %d1,(%a4)  
                bne.s   .-2
                move.b  %d4,(%a4)
                addq.w  #1,(%a2) 
                move.b  (%a0)+,(%a1)
                btst    %d1,(%a4)   
                bne.s   .-2
                move.b  %d4,(%a4)   
                addq.w  #1,(%a2)    
                move.b  (%a0)+,(%a1)  
                btst    %d1,(%a4)     
                bne.s   .-2
                move.b  %d4,(%a4)     
                addq.w  #1,(%a2)
                dbf     %d0,_gfx_txt0_r1
                add.w   40+22(%sp),%d3        |+chr offset
                bra.s   _gfx_txt0_u1
_gfx_txt0_e1:
                moveq   #-1,%d2
                move.l  %d2,(%a1)
                movem.l (%sp)+,%d0-%d4/%a0-%a4
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_copy_VRAM_block
                .type   gfx_copy_VRAM_block, @function
|   l_y. l_x. dy.. dx.. sy.. sx.. RRRR
|   +26  +22  +18  +14  +10  +6   sp
gfx_copy_VRAM_block:
                movem.l %d0-%d2/%a0-%a2,-(%sp)
                movea.l #GFX_COMMAND_STATUS_REGISTER,%a0
                movea.l #GFX_SOURCE_Y_REGISTER,%a1
                movea.l #GFX_DESTINATION_Y_REGISTER,%a2
                move.w  24+6(%sp),GFX_SOURCE_X_REGISTER
                move.w  24+10(%sp),GFX_SOURCE_Y_REGISTER
                move.w  24+14(%sp),GFX_DESTINATION_X_REGISTER
                move.w  24+18(%sp),GFX_DESTINATION_Y_REGISTER
                move.w  24+22(%sp),GFX_LENGTH_REGISTER
                move.w  24+26(%sp),%d0         | len_y
                subq.w  #1,%d0
                moveq   #-64,%d1              | = 0xC0 gfx opcode
                moveq   #7,%d2                |bit7 of CMD_STATUS_REG = ready
_gfx_vcp0_loop:
                btst    %d2,(%a0)             |wait gfx ready
                bne.s   .-2
                move.b  %d1,(%a0)             |load opcode, start
                nop
                addq.w  #1,(%a1)
                addq.w  #1,(%a2)
                dbf     %d0,_gfx_vcp0_loop
                movem.l (%sp)+,%d0-%d2/%a0-%a2
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  gfx_copy_picture_to_VRAM
                .type   gfx_copy_picture_to_VRAM, @function
| *ptr l_y. l_x. y... x... RRRR
| +20  +18  +14  +10  +6   sp

gfx_copy_picture_to_VRAM:
                movem.l %d0-%d2/%a0-%a2,-(%sp)
                move.w  24+10(%sp),%d0           | y
                andi.l  #511,%d0
                movea.l #VRAM_START_ADDRESS,%a2
                moveq   #10,%d1
                lsl.l   %d1,%d0
                lea     (%a2,%d0.l),%a2
                move.w  24+6(%sp),%d0            | x
                andi.w  #1023,%d0
                lea     (%a2,%d0.w),%a2          |get VRAM address
                movea.l 24+20(%sp),%a0           |*ptr
                move.w  24+18(%sp),%d1           | y_len
                beq.s   _gfx_cpypic_e
                subq.w  #1,%d1
                andi.w  #511,%d1              | y_loop_counter
                move.w  24+14(%sp),%d2           | x_len
                beq.s   _gfx_cpypic_e
                subq.w  #1,%d2
                andi.w  #1023,%d2
_gfx_cpypic_a0: 
                move.w  %d2,%d0
                movea.l %a2,%a1
_gfx_cpypic_a1:
                move.b  (%a0)+,(%a1)+
                dbf     %d0,_gfx_cpypic_a1
                lea     1024(%a2),%a2
                dbf     %d1,_gfx_cpypic_a0
_gfx_cpypic_e:
                movem.l (%sp)+,%d0-%d2/%a0-%a2
                rts
|----------------------------------------------------------------------------





