                .include "sys_regs.i"

|############################################################################
|                                   RS232
|############################################################################
                .section .text
                .align  2
                .globl  _rs232_initialize
_rs232_initialize:
                clr.b   RS232_RX_CONTROL_REGISTER
                clr.b   RS232_TX_CONTROL_REGISTER
                andi.w  #0xDDFF,INTERRUPT_MASK_REGISTER_L |mask irq9 and irq13
                move.w  #0x2200,INTERRUPT_POLL_CLEAR_REGISTER_L |clear irq's
                bsr     rs232_clear_rx_buf
                bsr     rs232_clear_tx_buf
                st      _rs232_tx_complete
                move.b  #0x80,RS232_RX_CONTROL_REGISTER   |enable rx interrupt
                ori.w   #0x2200,INTERRUPT_MASK_REGISTER_L |unmask irq's
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _rs232_rx_irq9_handler
_rs232_rx_irq9_handler:
                movem.l %d0/%a0,-(%sp)
                lea     _rs232_rx_buf,%a0
                move.w  _rs232_rx_wr_ptr,%d0
                move.b  RS232_RX_DATA_REGISTER,(%a0,%d0.w)
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_rs232_rx_wr_ptr
                movem.l (%sp)+,%d0/%a0
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _rs232_tx_irq13_handler
_rs232_tx_irq13_handler:
                movem.l %d0/%a0,-(%sp)
                move.w  _rs232_tx_rd_ptr,%d0
                lea     _rs232_tx_buf,%a0
                cmp.w   _rs232_tx_wr_ptr,%d0      | last byte transmitted?
                bne.s   _rs232_i13txh_go
                clr.b   RS232_TX_CONTROL_REGISTER | if last byte tx'ed ->stop
                st      _rs232_tx_complete
                bra.s   _rs232_i13txh_e
_rs232_i13txh_go:
                move.b  (%a0,%d0.w),RS232_TX_DATA_REGISTER
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_rs232_tx_rd_ptr
_rs232_i13txh_e:
                movem.l (%sp)+,%d0/%a0
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  rs232_clear_rx_buf
                .type   rs232_clear_rx_buf, @function
rs232_clear_rx_buf:
                clr.w   _rs232_rx_wr_ptr
                clr.w   _rs232_rx_rd_ptr
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  rs232_clear_tx_buf
                .type   rs232_clear_tx_buf, @function
rs232_clear_tx_buf:
                clr.w   _rs232_tx_wr_ptr
                clr.w   _rs232_tx_rd_ptr
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  rs232_is_rx_bytes
                .type   rs232_is_rx_bytes, @function
rs232_is_rx_bytes:
                move.w  _rs232_rx_wr_ptr,%d0
                cmp.w   _rs232_rx_rd_ptr,%d0
                sne     %d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  rs232_is_tx_complete
                .type   rs232_is_tx_complete, @function
rs232_is_tx_complete:
                move.b  _rs232_tx_complete,%d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  rs232_get_byte
                .type   rs232_get_byte, @function
rs232_get_byte:
                movem.l %d1/%a0,-(%sp)
                move.w  _rs232_rx_rd_ptr,%d1
                clr.l   %d0
                cmp.w   _rs232_rx_wr_ptr,%d1
                beq.s   _rs232_gby_e
                lea     _rs232_rx_buf,%a0
                move.b  (%a0,%d1.w),%d0
                addq.w  #1,%d1
                andi.w  #0x1FF,%d1
                move.w  %d1,_rs232_rx_rd_ptr
_rs232_gby_e:
                movem.l (%sp)+,%d1/%a0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  rs232_send_byte
                .type   rs232_send_byte, @function
rs232_send_byte:
                movem.l %d0/%a0,-(%sp)
                move    #0x2700,%sr
                btst    #7,RS232_TX_CONTROL_REGISTER    |TX IRQ enabled?
                bne.s   _rs232_sby_p
                clr.b   _rs232_tx_complete
                move.b  #0x80,RS232_TX_CONTROL_REGISTER |enable IRQ
                move.b  8+7(%sp),RS232_TX_DATA_REGISTER |and start TX
                bra.s   _rs232_sby_e
_rs232_sby_p:
                move.w  _rs232_tx_wr_ptr,%d0
                lea     _rs232_tx_buf,%a0
                move.b  8+7(%sp),(%a0,%d0.w)
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_rs232_tx_wr_ptr
_rs232_sby_e:
                move    #0x2500,%sr
                movem.l (%sp)+,%d0/%a0
                rts
|----------------------------------------------------------------------------

|############################################################################
|                                 MIDI - 1
|############################################################################
                .section .text
                .align  2
                .globl  _midi1_initialize
_midi1_initialize:
                clr.b   MIDI1_RX_CONTROL_REGISTER
                clr.b   MIDI1_TX_CONTROL_REGISTER
                andi.w  #0xBBFF,INTERRUPT_MASK_REGISTER_L |mask irq 10 and 14
                move.w  #0x4400,INTERRUPT_POLL_CLEAR_REGISTER_L |clear irq's
                bsr     midi1_clear_rx_buf
                bsr     midi1_clear_tx_buf
                st      _midi1_tx_complete
                move.b  #0x80,MIDI1_RX_CONTROL_REGISTER   |enable rx interrupt
                ori.w   #0x4400,INTERRUPT_MASK_REGISTER_L |unmask irq's
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _midi1_rx_irq10_handler
_midi1_rx_irq10_handler:
                movem.l %d0/%a0,-(%sp)
                lea     _midi1_rx_buf,%a0
                move.w  _midi1_rx_wr_ptr,%d0
                move.b  MIDI1_RX_DATA_REGISTER,(%a0,%d0.w)
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_midi1_rx_wr_ptr
                movem.l (%sp)+,%d0/%a0
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _midi1_tx_irq14_handler
_midi1_tx_irq14_handler:
                movem.l %d0/%a0,-(%sp)
                move.w  _midi1_tx_rd_ptr,%d0
                lea     _midi1_tx_buf,%a0
                cmp.w   _midi1_tx_wr_ptr,%d0      | last byte transmitted?
                bne.s   _midi1_i14txh_go
                clr.b   MIDI1_TX_CONTROL_REGISTER | if last byte tx'ed ->stop
                st      _midi1_tx_complete
                bra.s   _midi1_i14txh_e
_midi1_i14txh_go:
                move.b  (%a0,%d0.w),MIDI1_TX_DATA_REGISTER
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_midi1_tx_rd_ptr
_midi1_i14txh_e:
                movem.l (%sp)+,%d0/%a0
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi1_clear_rx_buf
                .type   midi1_clear_rx_buf, @function
midi1_clear_rx_buf:
                clr.w   _midi1_rx_wr_ptr
                clr.w   _midi1_rx_rd_ptr
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi1_clear_tx_buf
                .type   midi1_clear_tx_buf, @function
midi1_clear_tx_buf:
                clr.w   _midi1_tx_wr_ptr
                clr.w   _midi1_tx_rd_ptr
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi1_is_rx_bytes
                .type   midi1_is_rx_bytes, @function
midi1_is_rx_bytes:
                move.w  _midi1_rx_wr_ptr,%d0
                cmp.w   _midi1_rx_rd_ptr,%d0
                sne     %d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi1_is_tx_complete
                .type   midi1_is_tx_complete, @function
midi1_is_tx_complete:
                move.b  _midi1_tx_complete,%d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi1_get_byte
                .type   midi1_get_byte, @function
midi1_get_byte:
                movem.l %d1/%a0,-(%sp)
                move.w  _midi1_rx_rd_ptr,%d1
                clr.l   %d0
                cmp.w   _midi1_rx_wr_ptr,%d1
                beq.s   _midi1_gby_e
                lea     _midi1_rx_buf,%a0
                move.b  (%a0,%d1.w),%d0
                addq.w  #1,%d1
                andi.w  #0x1FF,%d1
                move.w  %d1,_midi1_rx_rd_ptr
_midi1_gby_e:
                movem.l (%sp)+,%d1/%a0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi1_send_byte
                .type   midi1_send_byte, @function
midi1_send_byte:
                movem.l %d0/%a0,-(%sp)
                move    #0x2700,%sr
                btst    #7,MIDI1_TX_CONTROL_REGISTER    |TX IRQ enabled?
                bne.s   _midi1_sby_p
                clr.b   _midi1_tx_complete
                move.b  #0x80,MIDI1_TX_CONTROL_REGISTER |enable IRQ
                move.b  8+7(%sp),MIDI1_TX_DATA_REGISTER |and start TX
                bra.s   _midi1_sby_e
_midi1_sby_p:
                move.w  _midi1_tx_wr_ptr,%d0
                lea     _midi1_tx_buf,%a0
                move.b  8+7(%sp),(%a0,%d0.w)
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_midi1_tx_wr_ptr
_midi1_sby_e:
                move    #0x2500,%sr
                movem.l (%sp)+,%d0/%a0
                rts
|----------------------------------------------------------------------------
|############################################################################
|                                 MIDI - 2
|############################################################################
                .section .text
                .align  2
                .globl  _midi2_initialize
_midi2_initialize:
                clr.b   MIDI2_RX_CONTROL_REGISTER
                clr.b   MIDI2_TX_CONTROL_REGISTER
           
                andi.w  #0x77FF,INTERRUPT_MASK_REGISTER_L |mask irq 11 and 15
                move.w  #0x8800,INTERRUPT_POLL_CLEAR_REGISTER_L |clear irq's
                bsr     midi2_clear_rx_buf
                bsr     midi2_clear_tx_buf
                st      _midi2_tx_complete
                move.b  #0x80,MIDI2_RX_CONTROL_REGISTER   |enable rx interrupt
                ori.w   #0x8800,INTERRUPT_MASK_REGISTER_L |unmask irq's
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _midi2_rx_irq11_handler
_midi2_rx_irq11_handler:
                movem.l %d0/%a0,-(%sp)
                lea     _midi2_rx_buf,%a0
                move.w  _midi2_rx_wr_ptr,%d0
                move.b  MIDI2_RX_DATA_REGISTER,(%a0,%d0.w)
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_midi2_rx_wr_ptr
                movem.l (%sp)+,%d0/%a0
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _midi2_tx_irq15_handler
_midi2_tx_irq15_handler:
                movem.l %d0/%a0,-(%sp)
                move.w  _midi2_tx_rd_ptr,%d0
                lea     _midi2_tx_buf,%a0
                cmp.w   _midi2_tx_wr_ptr,%d0      | last byte transmitted?
                bne.s   _midi2_i15txh_go
                clr.b   MIDI2_TX_CONTROL_REGISTER | if last byte tx'ed ->stop
                st      _midi2_tx_complete
                bra.s   _midi2_i15txh_e
_midi2_i15txh_go:
                move.b  (%a0,%d0.w),MIDI2_TX_DATA_REGISTER
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_midi2_tx_rd_ptr
_midi2_i15txh_e:
                movem.l (%sp)+,%d0/%a0
                rte
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi2_clear_rx_buf
                .type   midi2_clear_rx_buf, @function
midi2_clear_rx_buf:
                clr.w   _midi2_rx_wr_ptr
                clr.w   _midi2_rx_rd_ptr
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi2_clear_tx_buf
                .type   midi2_clear_tx_buf, @function
midi2_clear_tx_buf:
                clr.w   _midi2_tx_wr_ptr
                clr.w   _midi2_tx_rd_ptr
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi2_is_rx_bytes
                .type   midi2_is_rx_bytes, @function
midi2_is_rx_bytes:
                move.w  _midi2_rx_wr_ptr,%d0
                cmp.w   _midi2_rx_rd_ptr,%d0
                sne     %d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi2_is_tx_complete
                .type   midi2_is_tx_complete, @function
midi2_is_tx_complete:
                move.b  _midi2_tx_complete,%d0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi2_get_byte
                .type   midi2_get_byte, @function
midi2_get_byte:
                movem.l %d1/%a0,-(%sp)
                move.w  _midi2_rx_rd_ptr,%d1
                clr.l   %d0
                cmp.w   _midi2_rx_wr_ptr,%d1
                beq.s   _midi2_gby_e
                lea     _midi2_rx_buf,%a0
                move.b  (%a0,%d1.w),%d0
                addq.w  #1,%d1
                andi.w  #0x1FF,%d1
                move.w  %d1,_midi2_rx_rd_ptr
_midi2_gby_e:
                movem.l (%sp)+,%d1/%a0
                rts
|----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  midi2_send_byte
                .type   midi2_send_byte, @function
midi2_send_byte:
                movem.l %d0/%a0,-(%sp)
                move    #0x2700,%sr
                btst    #7,MIDI2_TX_CONTROL_REGISTER    |TX IRQ enabled?
                bne.s   _midi2_sby_p
                clr.b   _midi2_tx_complete
                move.b  #0x80,MIDI2_TX_CONTROL_REGISTER |enable IRQ
                move.b  8+7(%sp),MIDI2_TX_DATA_REGISTER |and start TX
                bra.s   _midi2_sby_e
_midi2_sby_p:
                move.w  _midi2_tx_wr_ptr,%d0
                lea     _midi2_tx_buf,%a0
                move.b  8+7(%sp),(%a0,%d0.w)
                addq.w  #1,%d0
                andi.w  #0x1FF,%d0
                move.w  %d0,_midi2_tx_wr_ptr
_midi2_sby_e:
                move    #0x2500,%sr
                movem.l (%sp)+,%d0/%a0
                rts

|----------------------------------------------------------------------------
                .align  2
_rs232_rx_wr_ptr:               ds.w  1
_rs232_rx_rd_ptr:               ds.w  1
_rs232_tx_wr_ptr:               ds.w  1
_rs232_tx_rd_ptr:               ds.w  1
_rs232_rx_buf:                  ds.b  512
_rs232_tx_buf:                  ds.b  512
_rs232_tx_complete:             ds.b  1
|----------------------------------------------------------------------------
                .align  2
_midi1_rx_wr_ptr:               ds.w  1
_midi1_rx_rd_ptr:               ds.w  1
_midi1_tx_wr_ptr:               ds.w  1
_midi1_tx_rd_ptr:               ds.w  1
_midi1_rx_buf:                  ds.b  512
_midi1_tx_buf:                  ds.b  512
_midi1_tx_complete:             ds.b  1
|----------------------------------------------------------------------------
                .align  2
_midi2_rx_wr_ptr:               ds.w  1
_midi2_rx_rd_ptr:               ds.w  1
_midi2_tx_wr_ptr:               ds.w  1
_midi2_tx_rd_ptr:               ds.w  1
_midi2_rx_buf:                  ds.b  512
_midi2_tx_buf:                  ds.b  512
_midi2_tx_complete:             ds.b  1
|----------------------------------------------------------------------------

